@@ -116,6 +116,7 @@ cpufreq_label_set_font (void)
 gboolean
 cpufreq_update_label (CpuInfo *cpu)
 {
+	GtkRequisition label_size;
 	gchar *label, *freq;
 	gint both;
 
@@ -147,6 +148,28 @@ cpufreq_update_label (CpuInfo *cpu)
 		else
 			gtk_label_set_angle (GTK_LABEL(cpuFreq->label), 0);
 		gtk_widget_show (cpuFreq->label);
+
+		/* Set label width to max width if smaller to avoid panel
+		   resizing/jumping (see bug #10385). */
+		gtk_widget_size_request (cpuFreq->label, &label_size);
+		if (cpuFreq->panel_mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
+			if (label_size.height < cpuFreq->label_max_width)
+				gtk_widget_set_size_request (GTK_WIDGET (cpuFreq->label),
+											 -1, cpuFreq->label_max_width);
+			else {
+				if (label_size.height > cpuFreq->label_max_width)
+					cpuFreq->layout_changed = TRUE;
+				cpuFreq->label_max_width = label_size.height;
+			}
+		else
+			if (label_size.width < cpuFreq->label_max_width)
+				gtk_widget_set_size_request (GTK_WIDGET (cpuFreq->label),
+											 cpuFreq->label_max_width, -1);
+			else {
+				if (label_size.width > cpuFreq->label_max_width)
+					cpuFreq->label_max_width = label_size.width;
+				cpuFreq->layout_changed = TRUE;
+			}
 	}
 	else
 	{
@@ -169,6 +192,7 @@ cpufreq_widgets_layout (void)
 						   !cpuFreq->options->show_label_governor);
 	gint pos = 1, lw = 0, lh = 0, iw = 0, ih = 0;
 
+	/* keep plugin small if label is hidden or user requested compact size */
 	small = (hide_label ? TRUE : cpuFreq->options->keep_compact);
 
 	switch (cpuFreq->panel_mode) {
@@ -182,14 +206,10 @@ cpufreq_widgets_layout (void)
 		break;
 	case XFCE_PANEL_PLUGIN_MODE_DESKBAR:
 		orientation = small ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
-		xfce_panel_plugin_set_small (cpuFreq->plugin, FALSE);
+		xfce_panel_plugin_set_small (cpuFreq->plugin, hide_label ? TRUE : FALSE);
 		break;
 	}
 
-	/* always set plugin small state when only icon is shown */
-	if (hide_label)
-		xfce_panel_plugin_set_small (cpuFreq->plugin, TRUE);
-
 	/* check if the label fits below the icon, else put them side by side */
 	if (GTK_IS_WIDGET(cpuFreq->label) && ! hide_label) {
 		gtk_widget_size_request (cpuFreq->label, &label_size);
@@ -201,6 +221,7 @@ cpufreq_widgets_layout (void)
 		iw = icon_size.width;
 		ih = icon_size.height;
 	}
+
 	if (cpuFreq->panel_mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL &&
 		orientation == GTK_ORIENTATION_VERTICAL &&
 		lh + ih + BORDER * 2 >= cpuFreq->panel_size) {
@@ -282,12 +303,19 @@ cpufreq_current_cpu ()
 }
 
 gboolean
-cpufreq_update_plugin (void)
+cpufreq_update_plugin (gboolean reset_label_size)
 {
 	CpuInfo *cpu;
 	gboolean ret;
 
 	cpu = cpufreq_current_cpu ();
+
+	if (reset_label_size) {
+		cpuFreq->label_max_width = -1;
+		gtk_widget_set_size_request (GTK_WIDGET (cpuFreq->label), -1, -1);
+		cpuFreq->layout_changed = TRUE;
+	}
+
 	ret = cpufreq_update_label (cpu);
 
 	if (cpuFreq->layout_changed) {
@@ -363,8 +391,7 @@ cpufreq_mode_changed (XfcePanelPlugin *plugin, XfcePanelPluginMode mode, CpuFreq
 {
 	cpuFreq->panel_mode = mode;
 	cpuFreq->panel_rows = xfce_panel_plugin_get_nrows (plugin);
-	cpuFreq->layout_changed = TRUE;
-	cpufreq_update_plugin ();
+	cpufreq_update_plugin (TRUE);
 }
 
 void
@@ -442,7 +469,7 @@ cpufreq_widgets (void)
 	gtk_widget_show (cpuFreq->box);
 	gtk_widget_show (cpuFreq->button);
 
-	cpufreq_update_plugin ();
+	cpufreq_update_plugin (TRUE);
 }
 
 static void
@@ -551,13 +578,14 @@ cpufreq_set_size (XfcePanelPlugin *plugin, gint size, CpuFreqPlugin *cpufreq)
 {
 	cpuFreq->panel_size = size;
 	cpuFreq->panel_rows = xfce_panel_plugin_get_nrows (plugin);
-	cpuFreq->layout_changed = TRUE;
+
 	cpufreq_update_icon (cpufreq);
-	cpufreq_update_plugin ();
+	cpufreq_update_plugin (TRUE);
 
 	return TRUE;
 }
 
+static void
 cpufreq_show_about(XfcePanelPlugin *plugin,
 				   CpuFreqPlugin *cpufreq)
 {
@@ -595,6 +623,7 @@ cpufreq_construct (XfcePanelPlugin *plugin)
 	cpuFreq->panel_mode = xfce_panel_plugin_get_mode (cpuFreq->plugin);
 	cpuFreq->panel_rows = xfce_panel_plugin_get_nrows (cpuFreq->plugin);
 	cpuFreq->panel_size = xfce_panel_plugin_get_size (cpuFreq->plugin);
+	cpuFreq->label_max_width = -1;
 	cpuFreq->cpus    = g_ptr_array_new ();
 
 	cpufreq_read_config ();
